@@ -26,8 +26,8 @@ class WFCDTD(DTDConsumer):
 	self.gen_ents={}
 	self.param_ents={}
 	self.elems={}
-        self.namespaces={}
         self.attrinfo={}
+        self.used_notations={} # Notations used by NOTATION attrs
 
 	# Adding predefined entities
 	for name in predef_ents.keys():
@@ -52,23 +52,30 @@ class WFCDTD(DTDConsumer):
 	element does not exist."""
 	return self.elems[name]
 
-    def get_namespace(self,prefix):
-        """Returns the namespace declared with this prefix. Throws a KeyError
-        if the prefix is not declared."""
-        return self.namespaces[prefix]
-
     def get_elements(self):
         "Returns a list of all declared element names."
         return self.elems.keys()
 
+    def get_notation(self,name):
+        """Returns the declaration of the notation. Throws KeyError if the
+        notation does not exist."""
+        raise KeyError(name)
+
+    def get_root_elem(self,name):
+        """Returns the name of the declared root element, or None if none
+        were declared."""
+        return None
+    
     # --- Shortcut information for validation
 
-    def prepare_for_parsing(self):
+    def dtd_end(self):
         "Stores shortcut information."
         self.attrinfo={}
         for elem in self.elems.values():
             self.attrinfo[elem.get_name()]=(elem.get_default_attributes(),
                                             elem.get_fixed_attributes())
+            
+        self.dtd_listener.dtd_end()
 
     def get_element_info(self,name):
         return self.attrinfo[name]
@@ -87,28 +94,41 @@ class WFCDTD(DTDConsumer):
     # --- Echoing DTD parse events
     
     def new_general_entity(self,name,val):
+        if self.gen_ents.has_key(name):
+            ## FIXME: May warn
+            return # Keep first decl
+        
         ent=InternalEntity(name,val)
-	self.gen_ents[ent.name]=ent
+	self.gen_ents[name]=ent
         self.dtd_listener.new_general_entity(name,val)
 
     def new_parameter_entity(self,name,val):
+        if self.param_ents.has_key(name):
+            ## FIXME: May warn
+            return # Keep first decl
+        
         ent=InternalEntity(name,val)
-	self.param_ents[ent.name]=ent
+	self.param_ents[name]=ent
         self.dtd_listener.new_parameter_entity(name,val)
 
     def new_external_entity(self,ent_name,pubid,sysid,ndata):
+        if self.gen_ents.has_key(ent_name):
+            ## FIXME: May warn
+            return # Keep first decl
+        
         ent=ExternalEntity(ent_name,pubid,sysid,ndata)
-	self.gen_ents[ent.name]=ent
+	self.gen_ents[ent_name]=ent
         self.dtd_listener.new_external_entity(ent_name,pubid,sysid,ndata)
 
     def new_external_pe(self,name,pubid,sysid):
+        if self.param_ents.has_key(name):
+            ## FIXME: May warn
+            return # Keep first decl
+        
         ent=ExternalEntity(name,pubid,sysid,"")
-	self.param_ents[ent.name]=ent
+	self.param_ents[name]=ent
         self.dtd_listener.new_external_pe(name,pubid,sysid)
 	
-    def dtd_end(self):
-        self.dtd_listener.dtd_end()
-    
     def new_comment(self,contents):
         self.dtd_listener.new_comment(contents)
 
@@ -150,11 +170,19 @@ class CompleteDTD(WFCDTD):
     # --- DTD parse events
 
     def dtd_end(self):
+        WFCDTD.dtd_end(self)
+        
 	for elem in self.attlists.keys():
 	    self.parser.report_error(1006,elem)
-
 	self.attlists={}  # Not needed any more, can free this memory
-        self.dtd_listener.dtd_end()
+
+        for notation in self.used_notations.keys():
+            try:
+                self.get_notation(notation)
+            except KeyError,e:
+                self.parser.report_error(2022,(self.used_notations[notation],
+                                               notation))
+        self.used_notations={} # Not needed, save memory            
 	
     def new_notation(self,name,pubid,sysid):
 	self.notations[name]=(pubid,sysid)
@@ -163,16 +191,19 @@ class CompleteDTD(WFCDTD):
     def new_element_type(self,elem_name,elem_cont):       
 	if self.elems.has_key(elem_name):
 	    self.parser.report_error(2012,elem_name)
+            return  # Keeping first declaration
 
-	if elem_cont==None:
+	if elem_cont=="EMPTY":
 	    model=make_empty_model()
-	elif elem_cont!=1:
-            model=make_model(elem_cont,self.parser)
+            elem_cont=("",[],"")
+	elif elem_cont!="ANY":
+            model=make_model(make_objects(elem_cont),self.parser)
+            elem_cont=None
 
-	if elem_cont==1:
+	if elem_cont=="ANY":
 	    self.elems[elem_name]=ElementTypeAny(elem_name)
 	else:
-	    self.elems[elem_name]=ElementType(elem_name,model)
+	    self.elems[elem_name]=ElementType(elem_name,model,elem_cont)
 
 	if self.attlists.has_key(elem_name):
 	    for (attr,a_type,a_decl,a_def) in self.attlists[elem_name]:
@@ -200,10 +231,11 @@ class CompleteDTD(WFCDTD):
 class ElementType:
     "Represents an element type."
 
-    def __init__(self,name,cont):
+    def __init__(self,name,compiled,original):
 	self.name=name
 	self.attrhash={}
-	self.content_model=cont
+	self.content_model=compiled
+        self.content_model_structure=original
 
     def get_name(self):
 	"Returns the name of the element type."
@@ -221,7 +253,8 @@ class ElementType:
 	"Adds a new attribute to the element."
 	if self.attrhash.has_key(attr):
 	    parser.report_error(1007,attr)
-	
+            return  # Keep first declaration
+
 	if a_type=="ID":
 	    for attr_name in self.attrhash.keys():
 		if self.attrhash[attr_name].type=="ID":
@@ -229,8 +262,14 @@ class ElementType:
 
 	    if a_decl!="#REQUIRED" and a_decl!="#IMPLIED":
 		parser.report_error(2014)
-	
+        elif type(a_type)==types.TupleType and a_type[0]=="NOTATION":
+            for notation in a_type[1]:
+                parser.dtd.used_notations[notation]=attr
+            
 	self.attrhash[attr]=Attribute(attr,a_type,a_decl,a_def)
+
+        if a_def!=None:
+            self.attrhash[attr].validate(self.attrhash[attr].default,parser)
 	
     def get_start_state(self):
 	"Return the start state of this content model."
@@ -251,6 +290,12 @@ class ElementType:
 
 	return 0
 
+    def get_content_model(self):
+        """Returns the element content model in (sep,cont,mod) format, where
+        cont is a list of (name,mod) and (sep,cont,mod) tuples. ANY content
+        models are represented as None, and EMPTYs as ("",[],"")."""
+        return self.content_model_structure
+    
     # --- Methods used to create shortcut validation information
 
     def get_default_attributes(self):
@@ -274,7 +319,7 @@ class ElementType:
 class ElementTypeAny(ElementType):
 
     def __init__(self,name):
-	ElementType.__init__(self,name,None)
+	ElementType.__init__(self,name,None,None)
 
     def get_start_state(self):
 	return 1
@@ -296,7 +341,12 @@ class Attribute:
 	self.name=name
 	self.type=attrtype
 	self.decl=decl
-	self.default=default
+
+        # Normalize the default value before setting it
+        if default!=None and self.type!="CDATA":
+            self.default=string.join(string.split(default))
+        else:
+            self.default=default
 
         # Handling code for special attribute xml:space
         
@@ -315,7 +365,7 @@ class Attribute:
                     error=1
 
             if error: parser.report_error(2016)                            
-                
+            
     def validate(self,value,parser):
 	"Validates given value for correctness."
 
@@ -392,25 +442,9 @@ class ExternalEntity:
         "Returns the system identifier of the entity."
         return self.sysid
 
-# ==============================
-# Namespaces
-# ==============================
-
-class Namespace:
-
-    def __init__(self,prefix,nsname,srcdef):
-        self.__prefix=prefix
-        self.__nsname=nsname
-        self.__srcdef=srcdef
-
-    def get_prefix(self):
-        return self.__prefix
-
-    def get_name(self):
-        return self.__nsname
-
-    def get_schema_URL(self):
-        return self.__srcdef
+    def get_notation(self):
+        "Returns the notation of the entity, or None if there is none."
+        return self.notation
 
 # ==============================
 # Internal classes
@@ -713,3 +747,21 @@ def make_model(content_model,err):
     builder=FNDABuilder()
     content_model.add_states(builder)
     return fnda2fda(builder.get_automaton(),builder.get_current_state(),err)
+
+def make_objects(content_model):
+    "Turns a tuple-ized content model into one based on objects."
+    (sep,contents,mod)=content_model
+
+    newconts=[]
+    for tup in contents:
+        if len(tup)==2:
+            newconts.append(ContentModel([tup[0]],tup[1]))
+        else:
+            newconts.append(make_objects(tup))
+    
+    if sep==",":
+        return SeqContentModel(newconts,mod)
+    elif sep=="|":
+        return ChoiceContentModel(newconts,mod)
+    elif sep=="":
+        return ContentModel(newconts,mod)
