@@ -1,10 +1,12 @@
 """
 Some common declarations for the xmlproc system gathered in one file.
 """
+
+# $Id: xmlutils.py,v 1.6 1999/02/10 01:46:03 amk Exp $
    
 import string,re,urlparse,os
 
-from xmlapp import *
+import xmlapp
 import errors
 
 # Standard exceptions
@@ -24,10 +26,10 @@ class EntityParser:
 
     def __init__(self):
         # --- Creating support objects
-	self.err=ErrorHandler(self)
-	self.ent=EntityHandler(self.err)
-        self.isf=InputSourceFactory()
-        self.pubres=PubIdResolver()
+	self.err=xmlapp.ErrorHandler(self)
+	self.ent=xmlapp.EntityHandler(self.err)
+        self.isf=xmlapp.InputSourceFactory()
+        self.pubres=xmlapp.PubIdResolver()
         self.data_charset="iso-8859-1"
         self.errors=errors.get_error_list("en")
         
@@ -89,10 +91,7 @@ class EntityParser:
 	    self.report_error(3000,sysID)
 	    return
 
-	self.ent_stack.append(self.get_current_sysid(),self.data,self.pos,\
-			      self.line,self.last_break,self.datasize,\
-			      self.last_upd_pos,self.block_offset)
-	
+	self._push_ent_stack()
 	self.current_sysID=sysID
 	self.pos=0
 	self.line=1
@@ -107,10 +106,7 @@ class EntityParser:
     def push_entity(self,sysID,contents):
 	"""Parse some text and consider it a new entity, making it possible
 	to return to the original entity later."""
-	self.ent_stack.append(self.get_current_sysid(),self.data,self.pos,\
-			      self.line,self.last_break,self.datasize,\
-			      self.last_upd_pos,self.block_offset)
-
+        self._push_ent_stack()
 	self.data=contents
 	self.current_sysID=sysID
 	self.pos=0
@@ -124,10 +120,7 @@ class EntityParser:
 
 	if self.ent_stack==[]: self.report_error(4000)
 
-	(self.current_sysID,self.data,self.pos,self.line,self.last_break,\
-	 self.datasize,self.last_upd_pos,self.block_offset)=self.ent_stack[-1]
-
-	del self.ent_stack[-1]
+        self._pop_ent_stack()
 	self.final=0
 	
     def read_from(self,fileobj,bufsize=16384):
@@ -204,10 +197,12 @@ class EntityParser:
 	"Parses any remnants of data in the last block."
 	if not self.pos+1==self.datasize:
 	    self.final=1
+            pos=self.pos
  	    try:
 		self.do_parse()
 	    except OutOfDataException,e:
-		self.report_error(3001)
+                if pos!=self.pos:
+                    self.report_error(3001)
                 
     # --- GENERAL UTILITY
     
@@ -369,7 +364,6 @@ class EntityParser:
         elif number<3000:
             self.err.error(msg)
         else:
-            # Make parser stop reporting data events!
             self.err.fatal(msg)
     
     #--- USEFUL METHODS
@@ -405,6 +399,219 @@ class EntityParser:
         (or document) entity is not considered external."""
         return self.ent_stack!=[] and \
                self.ent_stack[0][0]!=self.get_current_sysid()
+
+    # --- Internal methods
+
+    def _push_ent_stack(self):
+	self.ent_stack.append(self.get_current_sysid(),self.data,self.pos,\
+			      self.line,self.last_break,self.datasize,\
+			      self.last_upd_pos,self.block_offset,self.final)
+
+    def _pop_ent_stack(self):
+	(self.current_sysID,self.data,self.pos,self.line,self.last_break,\
+	 self.datasize,self.last_upd_pos,self.block_offset,self.final)=\
+         self.ent_stack[-1]
+	del self.ent_stack[-1]
+
+# ==============================
+# Common code for some parsers
+# ==============================
+
+class XMLCommonParser(EntityParser):
+
+    def parse_external_id(self,required=0,sysidreq=1):
+        """Parses an external ID declaration and returns a tuple consisting
+        of (pubid,sysid). If the required attribute is false neither SYSTEM
+        nor PUBLIC identifiers are required. If sysidreq is false a SYSTEM
+        identifier is not required after a PUBLIC one."""
+
+        pub_id=None
+        sys_id=None
+        
+	if self.now_at("SYSTEM"):
+	    self.skip_ws(1)
+	    sys_id=self.get_wrapped_match([("\"",reg_sysid_quote),\
+					   ("'",reg_sysid_apo)])
+	elif self.now_at("PUBLIC"):
+	    self.skip_ws(1)
+	    pub_id=self.get_wrapped_match([("\"",reg_pubid_quote),\
+					   ("'",reg_pubid_apo)])
+            pub_id=string.join(string.split(pub_id))
+
+            if sysidreq:
+                self.skip_ws(1)
+                sys_id=self.get_wrapped_match([("\"",reg_sysid_quote),\
+                                               ("'",reg_sysid_apo)])
+            else:
+                self.skip_ws()
+                if self.test_str("'") or self.test_str('"'):
+                    sys_id=self.get_wrapped_match([("\"",reg_sysid_quote),\
+                                                   ("'",reg_sysid_apo)])
+	else:
+            if required: self.report_error(3006)
+
+        return (pub_id,sys_id)
+
+    def __get_quoted_string(self):
+        "Returns the contents of a quoted string at current position."
+        try:
+            quo=self.data[self.pos]
+        except IndexError:
+            raise OutOfDataException()
+            
+        if not (self.now_at('"') or self.now_at("'")):
+            self.report_error(3004,("'\"'","'"))
+            self.scan_to(">")
+            return ""
+
+        return self.scan_to(quo)
+    
+    def parse_xml_decl(self,handler=None):
+	"Parses the contents of the XML declaration from after the '<?xml'."
+
+        textdecl=self.is_external() # If this is an external entity, then this
+                                    # is a text declaration, not an XML decl
+        
+	self.update_pos()
+	if self.get_column()!=5 or self.get_line()!=1 or \
+           (self.ent_stack!=[] and not textdecl):
+            if textdecl:
+                self.report_error(3007)
+            else:    
+                self.report_error(3008)                
+            
+	if self.seen_xmldecl: # Set in parse_pi, to avoid block problems
+            if textdecl:
+                self.report_error(3009)
+            else:
+                self.report_error(3010)
+
+	enc=None
+	sddecl=None
+        ver=None
+	self.skip_ws()
+        
+	if self.now_at("version"):
+	    self.skip_ws()
+	    if not self.now_at("="): self.report_error(3005,"=")
+	    self.skip_ws()
+            ver=self.__get_quoted_string()
+            if reg_ver.match(ver)==None:
+                self.report_error(3901)
+            
+	    if ver!="1.0":
+		self.report_error(1001)
+
+	    self.skip_ws()
+	elif not textdecl:
+            self.report_error(3011)
+
+	if self.now_at("encoding"):
+	    self.skip_ws()
+	    if not self.now_at("="): self.report_error(3005,"=")
+	    self.skip_ws()
+            enc=self.__get_quoted_string()
+            if reg_enc_name.match(enc)==None:
+                self.report_error(3902)
+
+            # Setting up correct conversion
+            if string.lower(enc)!="iso-8859-1":
+                self.report_error(1002,enc)
+            
+	    self.skip_ws()	    
+
+	if self.now_at("standalone"):
+            if textdecl:
+                self.report_error(3012)
+                sddecl="yes"
+            else:
+                self.skip_ws()
+                if not self.now_at("="): self.report_error(3005,"=")
+                self.skip_ws()
+                sddecl=self.__get_quoted_string()
+                if reg_std_alone.match(sddecl)==None:
+                    self.report_error(3911)
+                    
+                self.standalone= sddecl=="yes"
+
+                self.skip_ws()
+
+	self.skip_ws()
+
+        if handler!=None:
+            handler.set_entity_info(ver,enc,sddecl)
+
+    def parse_pi(self,handler,report_xml_decl=0):
+	"""Parses a processing instruction from after the '<?' to beyond
+	the '?>'."""
+	trgt=self.get_match(reg_name)
+
+	if trgt=="xml":
+            if report_xml_decl:
+                self.parse_xml_decl(handler)
+            else:
+                self.parse_xml_decl()
+                
+	    if not self.now_at("?>"):
+		self.report_error(3005,"?>")
+	    self.seen_xmldecl=1
+	else:
+	    if reg_res_pi.match(trgt)!=None:
+		if trgt=="xml:namespace":
+		    self.report_error(1003)
+                elif trgt!="xml-stylesheet":
+		    self.report_error(1004)
+
+            self.skip_ws()
+            rem=self.scan_to("?>") # OutOfDataException if not found
+            handler.handle_pi(trgt,rem)   
+
+    def parse_comment(self,handler):
+	"Parses the comment from after '<!--' to beyond '-->'."
+	handler.handle_comment(self.get_match(reg_comment_content))
+	if not self.now_at("-->"):
+	    self.report_error(3005,"-->")
+
+    def _read_char_ref(self):
+        "Parses a character reference and returns the character."
+        if self.now_at("x"):
+            digs=unhex(self.get_match(reg_hex_digits))
+        else:
+            digs=string.atoi(self.get_match(reg_digits))
+
+        if not (digs==9 or digs==10 or digs==13 or \
+                (digs>=32 and digs<=255)):
+            if digs>255:
+                self.report_error(1005,digs)
+            else:
+                self.report_error(3018,digs)
+            return ""
+        else:
+            return chr(digs)
+
+    def _get_name(self):
+        """Parses the name at the current position and returns it. An error
+        is reported if no name is present."""
+	if self.pos>self.datasize-5 and not self.final:
+	    raise OutOfDataException()
+
+        if self.data[self.pos] in namestart:
+            start=self.pos
+            self.pos=self.pos+1
+
+            try:
+                while self.data[self.pos] in namechars:
+                    self.pos=self.pos+1
+
+                return intern(self.data[start:self.pos])
+            except IndexError:
+                if self.final:
+                    return intern(self.data[start:])
+                else:
+                    raise OutOfDataException()
+        else:
+            self.report_error(3900)
+            return ""            
     
 # --- A collection of useful functions
 
@@ -434,7 +641,10 @@ def matches(regexp,str):
 
 def join_sysids(base,url):
     if urlparse.urlparse(base)[0]=="":
-        return os.path.join(os.path.split(base)[0],url)
+        if urlparse.urlparse(url)[0]!="":
+            return url
+        else:
+            return os.path.join(os.path.split(base)[0],url)
     else:
         return urlparse.urljoin(base,url)
 
@@ -468,7 +678,7 @@ reg_ent_val_quote=re.compile("[^\"]+")
 reg_ent_val_apo=re.compile("[^\']+")
 
 reg_attr_type=re.compile(r"CDATA|IDREFS|IDREF|ID|ENTITY|ENTITIES|NMTOKENS|"
-			 "NMTOKEN")
+			 "NMTOKEN") # NOTATION support separate
 reg_attr_def=re.compile(r"#REQUIRED|#IMPLIED")
 
 reg_digits=re.compile("[0-9]+")
@@ -476,7 +686,7 @@ reg_hex_digits=re.compile("[0-9a-fA-F]+")
 
 reg_res_pi=re.compile("xml",re.I)
 
-reg_int_dtd=re.compile("\"|'|<\\?|<!--|\\]>|<!\\[")
+reg_int_dtd=re.compile("\"|'|<\\?|<!--|\\]|<!\\[")
 
 reg_attval_stop_quote=re.compile("<|&|\"")
 reg_attval_stop_sing=re.compile("<|&|'")
@@ -484,6 +694,9 @@ reg_attval_stop_sing=re.compile("<|&|'")
 reg_decl_with_pe=re.compile("<(![^-\[]|\?)")
 reg_subst_pe_search=re.compile(">|%")
 
+reg_cond_sect=re.compile("<!\\[|\\]\\]>")
+reg_litval_stop=re.compile("%|&#")
+
 # RFC 1766 language codes
 
 reg_lang_code=re.compile("([a-zA-Z][a-zA-Z]|[iIxX]-([a-zA-Z])+)(-[a-zA-Z])*")
